/*==================================================================================================
*   Project              : RTD AUTOSAR 4.4
*   Platform             : CORTEXM
*   Peripheral           : 
*   Dependencies         : none
*
*   Autosar Version      : 4.4.0
*   Autosar Revision     : ASR_REL_4_4_REV_0000
*   Autosar Conf.Variant :
*   SW Version           : 0.9.0
*   Build Version        : S32K3_RTD_0_9_0_P02_D2107_ASR_REL_4_4_REV_0000_20210716
*
*   (c) Copyright 2020 - 2021 NXP Semiconductors
*   All Rights Reserved.
*
*   NXP Confidential. This software is owned or controlled by NXP and may only be
*   used strictly in accordance with the applicable license terms. By expressly
*   accepting such terms or by downloading, installing, activating and/or otherwise
*   using the software, you are agreeing that you have read, and that you agree to
*   comply with and are bound by, such license terms. If you do not agree to be
*   bound by the applicable license terms, then you may not retain, install,
*   activate or otherwise use the software.
==================================================================================================*/
/**
*   @file       MC_RGM.c
*   @version    0.9.0
*
*   @brief
*   @brief   POWER driver implementations.
*   @details POWER driver implementations.
*
*   @addtogroup POWER_DRIVER Power Ip Driver
*   @{
*/

#ifdef __cplusplus
extern "C"{
#endif


/**
* @page misra_violations MISRA-C:2012 violations
*
* @section MC_RGM_c_REF_1
* Violates MISRA 2012 Advisory Rule 20.1, #include directives should only be preceded by preprocessor
* directives or comments. AUTOSAR imposes the specification of the sections in which certain parts
* of the driver must be placed.
*
* @section Mcu_MC_RGM_c_REF_2
* Violates MISRA 2012 Required Directive 4.10, Precautions shall be taken in order to prevent the contents
* of a header file being included more than once. This comes from the order of includes in the .c file
* and from include dependencies. As a safe approach, any file must include all its dependencies.
* Header files are already protected against double inclusions. The inclusion of Mcu_MemMap.h is as
* per AUTOSAR requirement [SWS_MemMap_00003].
*
* @section Mcu_MC_RGM_c_REF_3
* Violates MISRA 2012 Required Rule 5.4, Macro identifiers shall be distinct.
* This violation is due to the AUTOSAR requirement [SWS_BSW_00036] (perform inter module checks).
*
* @section Mcu_MC_RGM_c_REF_4
* Violates MISRA 2012 Required Rule 5.1, External identifiers shall be distinct.
* This rule does not apply if either identifier is a macro identifier, because this case is covered
* by Rule 5.4 and Rule 5.5.
*
* @section Mcu_MC_RGM_c_REF_5
* Violates MISRA 2012 Required Rule 5.2, Identifiers declared in the same scope and name space shall be distinct.
* This rule does not apply if either identifier is a macro identifier, because this case is covered
* by Rule 5.4 and Rule 5.5.
*
* @section Mcu_MC_RGM_c_REF_6
* Violates MISRA 2012 Required Rule 5.5, Identifiers shall be distinct from macro names.
* The identifier clash is between macro identifiers. Therefore, this rule is incorrectly reported by the tool.
*
* @section Mcu_MC_RGM_c_REF_7
* Violates MISRA 2012 Required Rule 11.6, A cast shall not be performed between pointer to void and an
* arithmetic type.
* This is a false positive error. PC-Lint is checking against a more restrictive condition
* ("pointer/non-pointer to non-pointer/pointer" vs "pointer-to-void to arithmetic type").
*
* @section Mcu_MC_RGM_c_REF_8
* Violates MISRA 2012 Advisory Rule 11.4, A conversion should not be performed between a pointer to object
* and an integer type.
* The cast is used to access memory mapped registers.
*
* @section Mcu_MC_RGM_c_REF_9
* Violates MISRA 2012 Required Rule 10.3, The value of an expression shall not be assigned to an object with a
* narrower essential type or of a different essential type category.
* The assignment of "uint32" to "Mcu_ResetType" is done to convert the bits in FES/DES to a semantic reset reason.
*
* @section Mcu_MC_RGM_c_REF_10
* Violates MISRA 2012 Advisory Rule 10.5, The value of an expression should not be cast to an inappropriate
* essential type.
* The cast of "uint32" to "Mcu_ResetType" is done to convert the bits in FES/DES to a semantic reset reason.
*
* @section Mcu_MC_RGM_c_REF_11
* Violates MISRA 2012 Advisory Rule 17.8, A function parameter should not be modified.
* The modifications of these function parameters are correct and intended.
*
* @section Mcu_MC_RGM_c_REF_12
* Violates MISRA 2012 Advisory Rule 8.9, An object should be defined at block scope if its identifier only appears
* in a single function.
* The variable needs to be declared as global variable for using in the future.
*
* @section Mcu_MC_RGM_c_REF_13
* Violates MISRA 2012 Required Rule 10.8, The value of a composite expression shall not be cast to a different
* essential type category or a wider essential type.
* The cast (to a different essential type) is used to access memory mapped registers.
*
* @section Mcu_MC_RGM_c_REF_14
* Violates MISRA 2012 Required Rule 2.1, A project shall not contain unreachable code.
* Function-like macro with generic implementation resolved on platform-specific branch.
*
*/


/*==================================================================================================
                                         INCLUDE FILES
 1) system and project includes
 2) needed interfaces from external units
 3) internal and external interfaces from this unit
==================================================================================================*/
#include "Power_Ip_Private.h"

#if (defined(POWER_IP_ENABLE_USER_MODE_SUPPORT) && (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT))
  #if (defined(MCAL_MC_RGM_REG_PROT_AVAILABLE))
    #if (STD_ON == MCAL_MC_RGM_REG_PROT_AVAILABLE)
      #define USER_MODE_REG_PROT_ENABLED      (STD_ON)
      #include "RegLockMacros.h"
    #endif /* (STD_ON == MCAL_MC_RGM_REG_PROT_AVAILABLE) */
  #endif
#endif /* (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT) */

/*==================================================================================================
                               SOURCE FILE VERSION INFORMATION
==================================================================================================*/
#define MC_RGM_VENDOR_ID_C                      43
#define MC_RGM_AR_RELEASE_MAJOR_VERSION_C       4
#define MC_RGM_AR_RELEASE_MINOR_VERSION_C       4
#define MC_RGM_AR_RELEASE_REVISION_VERSION_C    0
#define MC_RGM_SW_MAJOR_VERSION_C               0
#define MC_RGM_SW_MINOR_VERSION_C               9
#define MC_RGM_SW_PATCH_VERSION_C               0

/*==================================================================================================
*                                     FILE VERSION CHECKS
==================================================================================================*/
/* Check if MC_RGM.c file and Power_Ip_Private.h file are of the same vendor */
#if (MC_RGM_VENDOR_ID_C != POWER_IP_PRIVATE_VENDOR_ID)
    #error "MC_RGM.c and Power_Ip_Private.h have different vendor ids"
#endif

/* Check if MC_RGM.c file and Power_Ip_Private.h file are of the same Autosar version */
#if ((MC_RGM_AR_RELEASE_MAJOR_VERSION_C != POWER_IP_PRIVATE_AR_RELEASE_MAJOR_VERSION) || \
     (MC_RGM_AR_RELEASE_MINOR_VERSION_C != POWER_IP_PRIVATE_AR_RELEASE_MINOR_VERSION) || \
     (MC_RGM_AR_RELEASE_REVISION_VERSION_C != POWER_IP_PRIVATE_AR_RELEASE_REVISION_VERSION) \
    )
    #error "AutoSar Version Numbers of MC_RGM.c and Power_Ip_Private.h are different"
#endif

/* Check if MC_RGM.c file and Power_Ip_Private.h file are of the same Software version */
#if ((MC_RGM_SW_MAJOR_VERSION_C != POWER_IP_PRIVATE_SW_MAJOR_VERSION) || \
     (MC_RGM_SW_MINOR_VERSION_C != POWER_IP_PRIVATE_SW_MINOR_VERSION) || \
     (MC_RGM_SW_PATCH_VERSION_C != POWER_IP_PRIVATE_SW_PATCH_VERSION) \
    )
    #error "Software Version Numbers of MC_RGM.c and Power_Ip_Private.h are different"
#endif

#if (defined(POWER_IP_ENABLE_USER_MODE_SUPPORT) && (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT))
  #if (defined(MCAL_MC_RGM_REG_PROT_AVAILABLE))
    #if (STD_ON == MCAL_MC_RGM_REG_PROT_AVAILABLE)
        #ifndef DISABLE_MCAL_INTERMODULE_ASR_CHECK
        /* Check if MC_RGM.c file and RegLockMacros.h file are of the same Autosar version */
            #if ((MC_RGM_AR_RELEASE_MAJOR_VERSION_C    != REGLOCKMACROS_AR_RELEASE_MAJOR_VERSION) || \
                (MC_RGM_AR_RELEASE_MINOR_VERSION_C    != REGLOCKMACROS_AR_RELEASE_MINOR_VERSION))
                #error "AutoSar Version Numbers of MC_RGM.c and RegLockMacros.h are different"
            #endif
        #endif
    #endif /* (STD_ON == MCAL_MC_RGM_REG_PROT_AVAILABLE) */
  #endif
#endif /* (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT) */
/*==================================================================================================
                          LOCAL TYPEDEFS (STRUCTURES, UNIONS, ENUMS)
==================================================================================================*/


/*==================================================================================================
                                       LOCAL MACROS
==================================================================================================*/


/*==================================================================================================
                                       LOCAL CONSTANTS
==================================================================================================*/


/*==================================================================================================
                                       LOCAL VARIABLES
==================================================================================================*/
#define MCU_START_SEC_VAR_INIT_32
/**
* @violates @ref MC_RGM_c_REF_1 #include directives should only be preceded by preprocessor
* directives or comments.
*
* @violates @ref MC_RGM_c_REF_2 Precautions shall be taken in order to prevent the contents
* of a header file being included more than once.
*/
#include "Mcu_MemMap.h"

static uint32 u32DesResetStatus = 0x00U;
static uint32 u32FesResetStatus = 0x00U;

#if (defined(FEATURE_RESET_DURING_STANDBY_SUPPORTED) && (FEATURE_RESET_DURING_STANDBY_SUPPORTED == STD_ON))
    #if (defined(MCU_ENTER_LOW_POWER_MODE) && (MCU_ENTER_LOW_POWER_MODE == STD_ON))
/** @violates @ref MC_RGM_c_REF_12 The variable needs to be declared as global variable for using in the future. */
static uint32 u32StandbyResetStatus = 0x00U;
    #endif /*(MCU_ENTER_LOW_POWER_MODE == STD_ON) */
#endif /* (FEATURE_RESET_DURING_STANDBY_SUPPORTED == STD_ON) */


#define MCU_STOP_SEC_VAR_INIT_32
/**
* @violates @ref MC_RGM_c_REF_1 #include directives should only be preceded by preprocessor
* directives or comments.
*
* @violates @ref MC_RGM_c_REF_2 Precautions shall be taken in order to prevent the contents
* of a header file being included more than once.
*/
#include "Mcu_MemMap.h"


#define MCU_START_SEC_VAR_INIT_UNSPECIFIED
/**
* @violates @ref MC_RGM_c_REF_1 #include directives should only be preceded by preprocessor
* directives or comments.
*
* @violates @ref MC_RGM_c_REF_2 Precautions shall be taken in order to prevent the contents
* of a header file being included more than once.
*/
#include "Mcu_MemMap.h"


static Power_Ip_MC_RGM_Type * pMC_RGM = (Power_Ip_MC_RGM_Type *)MC_RGM_BASE;

#if (defined(MCU_RESET_DOMAINS_SUPPORTED) && (MCU_RESET_DOMAINS_SUPPORTED == STD_ON))
static Power_Ip_RDC_Type * pRdc = (Power_Ip_RDC_Type *)RDC_BASE;
#endif /* (MCU_RESET_DOMAINS_SUPPORTED == STD_ON) */

#if (defined(MCU_RESET_ALTERNATE_ISR_USED) && (MCU_RESET_ALTERNATE_ISR_USED == STD_ON))
static volatile Power_MC_RGM_StatusType Power_MC_RGM_Status = POWER_MC_RGM_UNINIT;
#endif

#define MCU_STOP_SEC_VAR_INIT_UNSPECIFIED
/**
* @violates @ref MC_RGM_c_REF_1 #include directives should only be preceded by preprocessor
* directives or comments.
*
* @violates @ref MC_RGM_c_REF_2 Precautions shall be taken in order to prevent the contents
* of a header file being included more than once.
*/
#include "Mcu_MemMap.h"

/*==================================================================================================
                                       GLOBAL CONSTANTS
==================================================================================================*/


/*==================================================================================================
                                       GLOBAL VARIABLES
==================================================================================================*/


/*==================================================================================================
                                   LOCAL FUNCTION PROTOTYPES
==================================================================================================*/
#define MCU_START_SEC_CODE
/**
* @violates @ref Mcu_MC_RGM_c_REF_1 #include directives should only be preceded by preprocessor
* directives or comments.
*
* @violates @ref Mcu_MC_RGM_c_REF_2 Precautions shall be taken in order to prevent the contents
* of a header file being included more than once.
*/
#include "Mcu_MemMap.h"


static void MC_RGM_ClearFesResetFlags(uint32 u32RegValue);
static void MC_RGM_ClearDesResetFlags(uint32 u32RegValue);
static Power_Ip_ResetType MC_RGM_CheckResetReason(uint32 activeValue, uint8 resetMode, uint32 * position);
#if (defined(MCU_RESET_DOMAINS_SUPPORTED) && (MCU_RESET_DOMAINS_SUPPORTED == STD_ON))
static void MC_RGM_ConfigureCore(const MC_RGM_CoreConfigType * MC_RGM_pCoreConfigPtr, uint8 u8DomainIndex);
static void MC_RGM_CheckConfigureCore(const MC_RGM_CoreConfigType * MC_RGM_pCoreConfigPtr, uint8 u8DomainIndex);

static void MC_RGM_ConfigureCOFB(const MC_RGM_CofbConfigType * MC_RGM_pCofbConfigPtr, uint8 u8DomainIndex);
static void MC_RGM_CheckConfigureCOFB(const MC_RGM_CofbConfigType * MC_RGM_pCofbConfigPtr, uint8 u8DomainIndex);

static void MC_RGM_ConfigureResetDomainController(const MC_RGM_DomainConfigType * MC_RGM_pResetDomainConfigPtr);
static void MC_RGM_ReleaseDomainReset(uint8 u8DomainIndex);
static void MC_RGM_AssertDomainReset(uint8 u8DomainIndex);
static void Power_Ip_EnableResetDomain(const MC_RGM_DomainConfigType * MC_RGM_pResetDomainConfigPtr);
static void Power_Ip_DisableResetDomain(const MC_RGM_DomainConfigType * MC_RGM_pResetDomainConfigPtr);

#endif /* (MCU_RESET_DOMAINS_SUPPORTED == STD_ON) */

/*==================================================================================================
                                       LOCAL FUNCTIONS
==================================================================================================*/
/**
* @brief            This function clears all the reset flags from MC_RGM_FES.
* @details          This function clears all the reset flags from MC_RGM_FES.
*
* @param[in]        u32RegValue     The value of the MC_RGM_FES register.
*
* @return           void
*
*/
static void MC_RGM_ClearFesResetFlags(uint32 u32RegValue)
{
    uint32 StartTime;
    uint32 ElapsedTime;
    uint32 TimeoutTicks;
    boolean TimeoutOccurred = FALSE;
    uint32 u32RegValueTemp = u32RegValue;

    /* Clear all the reset flags of MC_RGM_FES */
    if ((uint32)0U != u32RegValueTemp)
    {
        /* Wait until the update has finished. */
        PowerStartTimeout(&StartTime, &ElapsedTime, &TimeoutTicks, POWER_TIMEOUT_VALUE_US);
        do
        {
            TimeoutOccurred = PowerTimeoutExpired(&StartTime, &ElapsedTime, TimeoutTicks);
            pMC_RGM->FES = u32RegValueTemp;

            u32RegValueTemp = pMC_RGM->FES & MC_RGM_FES_RWBITS_MASK32;
        }
        while ( ((uint32)0U != u32RegValueTemp) && (!TimeoutOccurred) );
        /* timeout notification */
        if(TimeoutOccurred)
        {
            ReportPowerErrors(POWER_IP_REPORT_TIMEOUT_ERROR, POWER_IP_ERR_CODE_RESERVED);
        }
    }
}

/**
* @brief            This function clears all the reset flags from MC_RGM_DES.
* @details          This function clears all the reset flags from MC_RGM_DES.
*                   Called by:
*                       - MC_RGM_GetResetReason().
*                       - MC_RGM_GetResetRawValue().
*
* @param[in]        u32RegValue     The value of the MC_RGM_DES register.
*
* @return           void
*
*/
static void MC_RGM_ClearDesResetFlags(uint32 u32RegValue)
{
    uint32 StartTime;
    uint32 ElapsedTime;
    uint32 TimeoutTicks;
    boolean TimeoutOccurred = FALSE;
    uint32 u32RegValueTemp = u32RegValue;

    /* Clear all the reset flags from MC_RGM_DES */
    if ((uint32)0U != u32RegValueTemp)
    {
        /* Wait until the update has finished. */
        PowerStartTimeout(&StartTime, &ElapsedTime, &TimeoutTicks, POWER_TIMEOUT_VALUE_US);
        do
        {
            TimeoutOccurred = PowerTimeoutExpired(&StartTime, &ElapsedTime, TimeoutTicks);
            pMC_RGM->DES = u32RegValueTemp;

            u32RegValueTemp = pMC_RGM->DES & MC_RGM_DES_RWBITS_MASK32;
        }
        while ( ((uint32)0x00U != u32RegValueTemp) && (!TimeoutOccurred) );
        /* Report error when timeout occurred */
        if(TimeoutOccurred)
        {
            ReportPowerErrors(POWER_IP_REPORT_TIMEOUT_ERROR, POWER_IP_ERR_CODE_RESERVED);
        }
    }
}

#if (defined(MCU_RESET_DOMAINS_SUPPORTED) && (MCU_RESET_DOMAINS_SUPPORTED == STD_ON))
static void MC_RGM_ReleaseDomainReset(uint8 u8DomainIndex)
{
    /* Get the domain out of reset */
    pMC_RGM->PRST[u8DomainIndex].PRST_COFB[0U] &= ~((uint32)(MC_RGM_PRST_0_PERIPH_0_RST_MASK));
}

static void MC_RGM_AssertDomainReset(uint8 u8DomainIndex)
{
    /* Place the domain into reset */
    pMC_RGM->PRST[u8DomainIndex].PRST_COFB[0U] |= MC_RGM_PRST_0_PERIPH_0_RST_MASK;
}

static void MC_RGM_ConfigureResetDomainController(const MC_RGM_DomainConfigType * MC_RGM_pResetDomainConfigPtr)
{
    /* Temporary variable for configure reset domain controller */
    uint32 tempValue = 0U;
    uint32 u32ResetDomainConfig = MC_RGM_pResetDomainConfigPtr->u32DomainRdcRegValue;
    uint32 u32ResetDomainStatus;
    uint32 StartTime;
    uint32 ElapsedTime;
    uint32 TimeoutTicks;
    boolean TimeoutOccurred = FALSE;
    uint8 u8DomainIndex = MC_RGM_pResetDomainConfigPtr->u8DomainIndex;

    /* Unlock the access to the register by setting RDC_RDn_CTRL_REG[CTRL_UNLOCK]. */
    tempValue = pRdc->RDC_CTRL_REG[u8DomainIndex];
    tempValue &= ~RESET_RD1_CTRL_REG_RD1_CTRL_UNLOCK_MASK;
    tempValue |= RDC_RDX_CTRL_REG_CTRL_UNLOCK;
    pRdc->RDC_CTRL_REG[u8DomainIndex] = tempValue;

    /* Write the new configuration for RDC_RDn_CTRL_REG[ICON_INTF_DIS_REQ_ACK]. */
    tempValue = pRdc->RDC_CTRL_REG[u8DomainIndex];
    tempValue &= ~RESET_RD1_CTRL_REG_RD1_INTERCONNECT_INTERFACE_DISABLE_MASK;
    tempValue |= u32ResetDomainConfig;
    pRdc->RDC_CTRL_REG[u8DomainIndex] = tempValue;

    /* Wait until the update has finished. */
    PowerStartTimeout(&StartTime, &ElapsedTime, &TimeoutTicks, POWER_TIMEOUT_VALUE_US);
    do
    {
        TimeoutOccurred = PowerTimeoutExpired(&StartTime, &ElapsedTime, TimeoutTicks);

        u32ResetDomainStatus = pRdc->RDC_STAT_REG[u8DomainIndex];
    } while ( ((RESET_RD1_STAT_REG_RD1_INTERCONNECT_INTERFACE_DISABLE_STAT(u32ResetDomainStatus)) != (RESET_RD1_CTRL_REG_RD1_INTERCONNECT_INTERFACE_DISABLE(u32ResetDomainConfig))) && (!TimeoutOccurred) );
    /* timeout notification */
    if(TimeoutOccurred)
    {
        ReportPowerErrors(POWER_IP_REPORT_TIMEOUT_ERROR, POWER_IP_ERR_CODE_RESERVED);
    }

    /* Lock the access to the register by clearing RDC_RDn_CTRL_REG[CTRL_UNLOCK]. */
    tempValue = pRdc->RDC_CTRL_REG[u8DomainIndex];
    tempValue &= ~RESET_RD1_CTRL_REG_RD1_CTRL_UNLOCK_MASK;
    tempValue |= RDC_RDX_CTRL_REG_CTRL_LOCK;
    pRdc->RDC_CTRL_REG[u8DomainIndex] = tempValue;
}

static void MC_RGM_ConfigureCOFB(const MC_RGM_CofbConfigType * MC_RGM_pCofbConfigPtr, uint8 u8DomainIndex)
{
    uint32 u32PeripheralsRstConfig   = MC_RGM_pCofbConfigPtr->u32CofbRstRegValue;
    uint32 u32PeripheralsRstToUpdate = MC_RGM_pCofbConfigPtr->u32CofbBlocksToUpdateMask;
    uint8 u8CofbIndex                = MC_RGM_pCofbConfigPtr->u8CofbIndex;
    uint32 tempValue                 = 0U;

    /* Write the new configuration for the COFB set. */
    tempValue = pMC_RGM->PRST[u8DomainIndex].PRST_COFB[u8CofbIndex];
    tempValue &= ~u32PeripheralsRstToUpdate;
    tempValue |= (u32PeripheralsRstConfig & u32PeripheralsRstToUpdate);
    pMC_RGM->PRST[u8DomainIndex].PRST_COFB[u8CofbIndex] = tempValue;
}

static void MC_RGM_CheckConfigureCOFB(const MC_RGM_CofbConfigType * MC_RGM_pCofbConfigPtr, uint8 u8DomainIndex)
{
    uint32 u32PeripheralsRstConfig   = MC_RGM_pCofbConfigPtr->u32CofbRstRegValue;
    uint32 u32PeripheralsRstToUpdate = MC_RGM_pCofbConfigPtr->u32CofbBlocksToUpdateMask;
    uint8 u8CofbIndex                = MC_RGM_pCofbConfigPtr->u8CofbIndex;
    uint32 u32PeripheralsRstStatus;
    uint32 StartTime;
    uint32 ElapsedTime;
    uint32 TimeoutTicks;
    boolean TimeoutOccurred = FALSE;

    /* Wait until the update has finished. */
    PowerStartTimeout(&StartTime, &ElapsedTime, &TimeoutTicks, POWER_TIMEOUT_VALUE_US);
    do
    {
        TimeoutOccurred = PowerTimeoutExpired(&StartTime, &ElapsedTime, TimeoutTicks);

        u32PeripheralsRstStatus = pMC_RGM->PRST[u8DomainIndex].PRST_COFB[u8CofbIndex];
    } while ( ((u32PeripheralsRstConfig & u32PeripheralsRstToUpdate) != (u32PeripheralsRstStatus & u32PeripheralsRstToUpdate)) && (!TimeoutOccurred ));
    /* timeout notification */
    if(TimeoutOccurred)
    {
        ReportPowerErrors(POWER_IP_REPORT_TIMEOUT_ERROR, POWER_IP_ERR_CODE_RESERVED);
    }
}


static void MC_RGM_ConfigureCore(const MC_RGM_CoreConfigType * MC_RGM_pCoreConfigPtr, uint8 u8DomainIndex)
{
    uint32 u32CoreConfig   = MC_RGM_pCoreConfigPtr->u32CorePrstRegValue;
    uint32 u32CoreToUpdate = MC_RGM_pCoreConfigPtr->u32CoreBlocksToUpdateMask;
    uint32 tempValue = 0U;

    /* Write the new configuration for the core. */
    tempValue = pMC_RGM->PRST[u8DomainIndex].PRST_COFB[0U];
    tempValue &= ~u32CoreToUpdate;
    tempValue |= (u32CoreConfig & u32CoreToUpdate);
    pMC_RGM->PRST[u8DomainIndex].PRST_COFB[0U] = tempValue;
}

static void MC_RGM_CheckConfigureCore(const MC_RGM_CoreConfigType * MC_RGM_pCoreConfigPtr, uint8 u8DomainIndex)
{
    uint32 u32CoreConfig   = MC_RGM_pCoreConfigPtr->u32CorePrstRegValue;
    uint32 u32CoreToUpdate = MC_RGM_pCoreConfigPtr->u32CoreBlocksToUpdateMask;
    uint32 u32CoreStatus;
    uint32 StartTime;
    uint32 ElapsedTime;
    uint32 TimeoutTicks;
    boolean TimeoutOccurred = FALSE;

    /* Wait until the update has finished. */
    PowerStartTimeout(&StartTime, &ElapsedTime, &TimeoutTicks, POWER_TIMEOUT_VALUE_US);
    do
    {
        TimeoutOccurred = PowerTimeoutExpired(&StartTime, &ElapsedTime, TimeoutTicks);

        u32CoreStatus = pMC_RGM->PRST[u8DomainIndex].PRST_COFB[0U];
    } while ( ((u32CoreStatus & u32CoreToUpdate) != (u32CoreConfig & u32CoreToUpdate)) && (!TimeoutOccurred) );
    /* timeout notification */
    if(TimeoutOccurred)
    {
        ReportPowerErrors(POWER_IP_REPORT_TIMEOUT_ERROR, POWER_IP_ERR_CODE_RESERVED);
    }
}
#endif /* (MCU_RESET_DOMAINS_SUPPORTED == STD_ON) */

/*==================================================================================================
                                       GLOBAL FUNCTIONS
==================================================================================================*/

#if (defined(POWER_IP_ENABLE_USER_MODE_SUPPORT) && (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT))
  #if (defined(MCAL_MC_RGM_REG_PROT_AVAILABLE))
    #if (STD_ON == MCAL_MC_RGM_REG_PROT_AVAILABLE)
/**
* @brief            This function will enable writing in User mode by configuring REG_PROT
*/
void MC_RGM_SetUserAccessAllowed(void)
{
#if (defined(MC_RGM_BASE))
    SET_USER_ACCESS_ALLOWED(MC_RGM_BASE, MC_RGM_PROT_MEM_U32);
#endif
}
    #endif /* (STD_ON == MCAL_MC_RGM_REG_PROT_AVAILABLE) */
  #endif
#endif /* (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT) */


#if (defined(POWER_IP_ENABLE_USER_MODE_SUPPORT) && (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT))
  #if (defined(MCAL_RDC_REG_PROT_AVAILABLE))
    #if(STD_ON == MCAL_RDC_REG_PROT_AVAILABLE)
/**
* @brief            This function will enable writing in User mode by configuring REG_PROT
*/
void RDC_SetUserAccessAllowed(void)
{
#if (defined(RDC_BASE))
    SET_USER_ACCESS_ALLOWED(RDC_BASE, RDC_PROT_MEM_U32);
#endif
}
    #endif
  #endif
#endif /* (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT) */


/**
* @brief            This function initializes the Reset parameters.
* @details          This function configures disabling the functional event reset, disabling the
*                   destructive event reset, the functional event alternate request, the
*                   destructive event alternate request, the functional event short sequence, the
*                   standby reset sequence and enabling the functional bidirectional reset.
*
* @param[in]        MC_RGM_pConfigPtr   Pointer to the MC_RGM configuration structure.
*
* @return           void
*
*/
void MC_RGM_ResetInit(const Power_Ip_MC_RGM_ConfigType * MC_RGM_pConfigPtr)
{
    /* Temporary variables for FES and DES register values. */
    uint32 u32FesResetReason = 0U;
    uint32 u32DesResetReason = 0U;


    /* Warning: It is important to clear the RGM_FES register before setting any of the bits
     * in the RGM_FERD register to '1'. Otherwise, a redundant interrupt request may occur.
     * Also, each byte in RGM_FERD can be written only once after a power-on reset. */

    /* Check "Functional Reset Reason" from FES Status Register. */
    u32FesResetReason = pMC_RGM->FES;

    /* Store the contents of FES. */
    if ((uint32)0U != u32FesResetReason)
    {
        u32FesResetStatus = u32FesResetReason;
        MC_RGM_ClearFesResetFlags(u32FesResetReason);
    }

#if (defined(POWER_IP_FUNCTIONAL_RESET_DISABLE_SUPPORT) && (POWER_IP_FUNCTIONAL_RESET_DISABLE_SUPPORT == STD_ON))
    /* 'Functional' Event Reset Disable Register (RGM_FERD) */
    pMC_RGM->FERD = MC_RGM_pConfigPtr->u32FuncResetOpt;
#endif /* POWER_IP_FUNCTIONAL_RESET_DISABLE_SUPPORT */

    /* Check "Destructive Reset Reason" from DES Status Register. */
    u32DesResetReason = pMC_RGM->DES;

    /* Enable (reconfigure) FRET only on "Power On Reset" or "Destructive Reset" event.   */
    /* Otherwise, the "Functional Reset Escalation" will not work, because the "internal  */
    /* functional reset counter" will be reset to 0 each time "Mcu_Init" is called.       */
    /* Therefore, it will never reach the functional threshold value.                     */
    if ((uint32)0U != u32DesResetReason)
    {
        /* 'Functional' Reset Escalation Threshold Register (MC_RGM_FRET) */
        pMC_RGM->FRET = MC_RGM_pConfigPtr->u32FesThresholdReset;
    }

    /* Enable (reconfigure) DRET only on "Power On Reset" event.                          */
    /* Otherwise, the "Destructive Reset Escalation" will not work, because the "internal */
    /* destructive reset counter" will be reset to 0 each time "Mcu_Init" is called.      */
    /* Therefore, it will never reach the destructive threshold value.                    */
    if (MC_RGM_DES_F_POR_MASK == (u32DesResetReason & MC_RGM_DES_F_POR_MASK))
    {
        /* 'Destructive' Reset Escalation Threshold Register (MC_RGM_DRET) */
        pMC_RGM->DRET = MC_RGM_pConfigPtr->u32DesThresholdReset;
    }
#if (defined(MCU_RESET_ALTERNATE_ISR_USED) && (MCU_RESET_ALTERNATE_ISR_USED == STD_ON))
    Power_MC_RGM_Status = POWER_MC_RGM_INIT;
#endif
}


#if (defined(MCU_RESET_DOMAINS_SUPPORTED) && (MCU_RESET_DOMAINS_SUPPORTED == STD_ON))
static void Power_Ip_EnableResetDomain(const MC_RGM_DomainConfigType * MC_RGM_pResetDomainConfigPtr)
{
    /* Only Software Resettable Partitions (Non-Main Reset Partitions) have RDC entries */
    if (MC_RGM_PRST_MAIN_PRST_INDEX != MC_RGM_pResetDomainConfigPtr->u8DomainIndex)
    {
        if ( (boolean)TRUE == MC_RGM_pResetDomainConfigPtr->bDomainPowerUnderMcuControl )
        {
            /* The reset domain must be enabled before configuring its COFBs and Cores */
            if (RDC_RDX_CTRL_REG_ICON_INTF_ENABLE == (MC_RGM_pResetDomainConfigPtr->u32DomainRdcRegValue & RESET_RD1_CTRL_REG_RD1_INTERCONNECT_INTERFACE_DISABLE_MASK))
            {
                MC_RGM_ConfigureResetDomainController(MC_RGM_pResetDomainConfigPtr);
                MC_RGM_ReleaseDomainReset(MC_RGM_pResetDomainConfigPtr->u8DomainIndex);
            }
        }
    }
    else
    {
        /* Do nothing */
    }
}
static void Power_Ip_DisableResetDomain(const MC_RGM_DomainConfigType * MC_RGM_pResetDomainConfigPtr)
{
    /* Only Software Resettable Partitions (Non-Main Reset Partitions) have RDC entries */
    if (MC_RGM_PRST_MAIN_PRST_INDEX != MC_RGM_pResetDomainConfigPtr->u8DomainIndex)
    {
        if ( (boolean)TRUE == MC_RGM_pResetDomainConfigPtr->bDomainPowerUnderMcuControl )
        {
            /* The reset domain must be disabled after configuring its COFBs and Cores */
            if (RDC_RDX_CTRL_REG_ICON_INTF_DISABLE == (MC_RGM_pResetDomainConfigPtr->u32DomainRdcRegValue & RESET_RD1_CTRL_REG_RD1_INTERCONNECT_INTERFACE_DISABLE_MASK))
            {
                MC_RGM_AssertDomainReset(MC_RGM_pResetDomainConfigPtr->u8DomainIndex);
                MC_RGM_ConfigureResetDomainController(MC_RGM_pResetDomainConfigPtr);
            }
        }
    }
    else
    {
        /* Do nothing */
    }
}
void Power_Ip_MC_RGM_ModeConfig(const Power_Ip_MC_RGM_ModeConfigType * MC_RGM_pModeConfigPtr)
{
    const MC_RGM_DomainConfigType *pTempDomainConfig = NULL_PTR;
    uint8 u8DomainIndex;
    uint8 u8CofbIndex;
    uint8 u8CoreIndex;

    for (u8DomainIndex = 0U; u8DomainIndex < POWER_IP_MAX_NUMBER_OF_PARTITIONS; ++u8DomainIndex)
    {
        pTempDomainConfig = &(*MC_RGM_pModeConfigPtr->apDomainConfig)[u8DomainIndex];

        if ( (boolean)TRUE == pTempDomainConfig->bDomainUnderMcuControl )
        {
            /* Only Software Resettable Partitions (Non-Main Reset Partitions) have RDC entries */
            Power_Ip_EnableResetDomain(pTempDomainConfig);

            for (u8CofbIndex = 0U; u8CofbIndex < pTempDomainConfig->u8NumberOfCofbs; ++u8CofbIndex)
            {
                if ( (boolean)TRUE == (*pTempDomainConfig->apDomainCofbConfig)[u8CofbIndex].bCofbUnderMcuControl )
                {
                    MC_RGM_ConfigureCOFB(&(*pTempDomainConfig->apDomainCofbConfig)[u8CofbIndex], pTempDomainConfig->u8DomainIndex);
                }
            }

            for (u8CoreIndex = 0U; u8CoreIndex < pTempDomainConfig->u8NumberOfCores; ++u8CoreIndex)
            {
                if ( (boolean)TRUE == (*pTempDomainConfig->apDomainCoreConfig)[u8CoreIndex].bCoreUnderMcuControl )
                {
                    MC_RGM_ConfigureCore(&(*pTempDomainConfig->apDomainCoreConfig)[u8CoreIndex], pTempDomainConfig->u8DomainIndex);
                }
            }

            /* Only Software Resettable Partitions (Non-Main Reset Partitions) have RDC entries */
            Power_Ip_DisableResetDomain(pTempDomainConfig);
        }
    }
}


void Power_Ip_MC_RGM_CheckModeConfig(const Power_Ip_MC_RGM_ModeConfigType * MC_RGM_pModeConfigPtr)
{
    const MC_RGM_DomainConfigType *pTempDomainConfig = NULL_PTR;
    uint8 u8DomainIndex;
    uint8 u8CofbIndex;
    uint8 u8CoreIndex;

    for (u8DomainIndex = 0U; u8DomainIndex < POWER_IP_MAX_NUMBER_OF_PARTITIONS; ++u8DomainIndex)
    {
        pTempDomainConfig = &(*MC_RGM_pModeConfigPtr->apDomainConfig)[u8DomainIndex];

        if ( (boolean)TRUE == pTempDomainConfig->bDomainUnderMcuControl )
        {
            for (u8CofbIndex = 0U; u8CofbIndex < pTempDomainConfig->u8NumberOfCofbs; ++u8CofbIndex)
            {
                if ( (boolean)TRUE == (*pTempDomainConfig->apDomainCofbConfig)[u8CofbIndex].bCofbUnderMcuControl )
                {
                    MC_RGM_CheckConfigureCOFB(&(*pTempDomainConfig->apDomainCofbConfig)[u8CofbIndex], pTempDomainConfig->u8DomainIndex);
                }
            }

            for (u8CoreIndex = 0U; u8CoreIndex < pTempDomainConfig->u8NumberOfCores; ++u8CoreIndex)
            {
                if ( (boolean)TRUE == (*pTempDomainConfig->apDomainCoreConfig)[u8CoreIndex].bCoreUnderMcuControl )
                {
                    MC_RGM_CheckConfigureCore(&(*pTempDomainConfig->apDomainCoreConfig)[u8CoreIndex], pTempDomainConfig->u8DomainIndex);
                }
            }
        }
    }
}
#endif /* (MCU_RESET_DOMAINS_SUPPORTED == STD_ON) */

static Power_Ip_ResetType MC_RGM_CheckResetReason(uint32 activeValue, uint8 resetMode, uint32 * position)
{
    /* Temporary variables for DES and FES registers value. */
    Power_Ip_ResetType eResetReason = MCU_NO_RESET_REASON;
    uint32 resetMask;
    uint32 u32NumberOfFlags = 0U;
    uint32 u32Index;
    uint32 u32DynamicMask;
    uint32 u32Position = *position;

    /* Type 0 : destructive reset
       Type 1 : functional reset */
    if (resetMode == 0U)
    {
        resetMask = MC_RGM_DES_RWBITS_MASK32;
    }
    else
    {
        resetMask = MC_RGM_FES_RWBITS_MASK32;
    }

    for (u32Index = 0U; u32Index < (uint32)32U; u32Index++)
    {
        u32DynamicMask = ((uint32)0x01U << u32Index);
        if ( (uint32)0U != (u32DynamicMask & resetMask) )
        {
            if ( (uint32)0U != (u32DynamicMask & activeValue) )
            {
                eResetReason = (Power_Ip_ResetType)u32Position;
                u32NumberOfFlags++;

                /*
                   MCU_MULTIPLE_RESET_REASON returned if more than one reset reason.
                   In this case, use function Mcu_GetRawValue to determine.
                */
                if (2U <= u32NumberOfFlags)
                {
                    eResetReason = MCU_MULTIPLE_RESET_REASON;
                    break;
                }
            }

            u32Position++;
        }
    }
    /* update value for position. It used to calculate in both Des and Fes */
    *position = u32Position;
    (void)u32NumberOfFlags;
    (void)activeValue;

    return eResetReason;
}
/**
* @brief            This function returns the Reset reason.
* @details          This routine returns the Reset reason that is read from the hardware.
*                   Called by:
*                       - Mcu_Ipw_GetResetReason() from IPW.
*
* @return           Reason of the Reset event.
* @retval           Power_Ip_ResetType   Platform-specific enumeration defined in "".
*
* @note             The User should ensure that the reset reason is cleared once it has been read
*                   out to avoid multiple reset reasons. The function Mcu_GetResetReason shall
*                   return MCU_RESET_UNDEFINED if this function is called prior to calling of the
*                   function Mcu_Init, and if supported by the hardware.
*/
Power_Ip_ResetType MC_RGM_GetResetReason(void)
{
    /* Code for the Reset event returned by this function. */
    Power_Ip_ResetType eResetReason = MCU_NO_RESET_REASON;

    /* Temporary variables for DES and FES registers value. */
    uint32 u32RegValue = 0U;
    uint32 u32ActiveValue;
    uint32 u32Position = MCU_FIRST_RESET_REASON_POS;

    /* Check "Destructive Reset Reason" from DES Status Register. */
    u32RegValue = pMC_RGM->DES & MC_RGM_DES_RWBITS_MASK32;

    /* Store the contents of DES. */
    if ((uint32)0U != u32RegValue)
    {
        u32DesResetStatus = u32RegValue;
    }

    u32ActiveValue = u32DesResetStatus;
    /* If at least one bit is set in the RGM_DES register, then it means that some destructive reset event has been logged. */
    eResetReason = MC_RGM_CheckResetReason(u32ActiveValue, 0U, &u32Position);

    /* Clear all the reset flags from MC_RGM_DES. */
    MC_RGM_ClearDesResetFlags(u32RegValue);

    if (MCU_MULTIPLE_RESET_REASON != eResetReason)
    {
        /* Check "Functional Reset Reason" from FES Status Register. */
        u32RegValue = pMC_RGM->FES & MC_RGM_FES_RWBITS_MASK32;

        /* Store the contents of FES. */
        if ((uint32)0U != u32RegValue)
        {
            u32FesResetStatus = u32RegValue;
        }

        u32ActiveValue = u32FesResetStatus;
        if ((uint32)0U != u32ActiveValue)
        {
            /* If at least one bit is set in the RGM_FES register, then it means that some functional reset event has been logged. */
            eResetReason = MC_RGM_CheckResetReason(u32ActiveValue, 1U, &u32Position);
        }

        /* Clear all the reset flags from MC_RGM_FES */
        MC_RGM_ClearFesResetFlags(u32RegValue);
    }

    return eResetReason;
}


/**
* @brief            This function returns the Raw Reset value.
* @details          This routine returns the Raw Reset value that is read from the hardware.
*
* @return           Implementation-specific value with the Reset status.
* @retval           Power_Ip_RawResetType   Code of the Raw reset value.
*
* @note             The User should ensure that the reset reason is cleared once it has been read
*                   out to avoid multiple reset reasons.
*
*/
Power_Ip_RawResetType MC_RGM_GetResetRawValue(void)
{
    /* Temporary variables for DES and FES registers value. */
    uint32 u32RawReset = 0U;
    uint32 u32RegValue;
    uint32 u32ActiveValue;
    uint32 u32Index;
    uint32 u32DynamicMask;
    uint32 u32Position;
#if (defined(MCU_ENTER_LOW_POWER_MODE) && (MCU_ENTER_LOW_POWER_MODE == STD_ON))
    Power_Ip_PowerModeType ePreviousMode;
#endif

    /* Compact the 2x32 registers to 1x32 register. */
    /* Lower 16 bits will be FES, higher 16 bits will be DES. */
    u32RegValue = pMC_RGM->FES & MC_RGM_FES_RWBITS_MASK32;

    /* Store the contents of FES. */
    if ((uint32)0U != u32RegValue)
    {
        u32FesResetStatus = u32RegValue;
    }

    u32ActiveValue = u32FesResetStatus;
    if ((uint32)0U != u32ActiveValue)
    {
        u32Position = MC_RGM_RAWRESET_FES_BASE_SHIFT_U32;

        for (u32Index = 0U; u32Index < (uint32)32U; u32Index++)
        {
            u32DynamicMask = ((uint32)0x01U << u32Index);
            if ( (uint32)0U != (u32DynamicMask & MC_RGM_FES_RWBITS_MASK32) )
            {
                if ( (uint32)0U != (u32DynamicMask & u32ActiveValue) )
                {
                    u32RawReset |= ((uint32)0x01U << u32Position);
                }

                u32Position++;
            }
        }

        /* Clear all the reset flags of MC_RGM_FES. */
        MC_RGM_ClearFesResetFlags(u32RegValue);
    }

    u32RegValue = pMC_RGM->DES & MC_RGM_DES_RWBITS_MASK32;

    /* Store the contents of DES. */
    if ((uint32)0U != u32RegValue)
    {
        u32DesResetStatus = u32RegValue;
    }

    u32ActiveValue = u32DesResetStatus;
    if ((uint32)0U != u32ActiveValue)
    {
        u32Position = MC_RGM_RAWRESET_DES_BASE_SHIFT_U32;

        for (u32Index = 0x00U; u32Index < (uint32)32U; u32Index++)
        {
            u32DynamicMask = ((uint32)0x01U << u32Index);
            if ( (uint32)0x00U != (u32DynamicMask & MC_RGM_DES_RWBITS_MASK32) )
            {
                if ( (uint32)0x00U != (u32DynamicMask & u32ActiveValue) )
                {
                    u32RawReset |= ((uint32)0x01U << u32Position);
                }

                u32Position++;
            }
        }

        /* Clear all the reset flags from MC_RGM_DES. */
        MC_RGM_ClearDesResetFlags(u32RegValue);
    }

#if (defined(MCU_ENTER_LOW_POWER_MODE) && (MCU_ENTER_LOW_POWER_MODE == STD_ON))
    /* If there is no reset reason on the main platform */
    if ((uint32)0U == u32RawReset)
    {
        ePreviousMode = Power_Ip_MC_ME_GetPreviousMode();
        if ( POWER_IP_SOC_STANDBY_MODE == ePreviousMode )
        {
            u32RawReset |= MC_RGM_RAWRESET_WAKEUP_MASK32;
        }
    }
#endif /*(MCU_ENTER_LOW_POWER_MODE == STD_ON) */

    return (Power_Ip_RawResetType)u32RawReset;
}

#if (defined(FEATURE_RESET_DURING_STANDBY_SUPPORTED) && (FEATURE_RESET_DURING_STANDBY_SUPPORTED == STD_ON))
    #if (defined(MCU_ENTER_LOW_POWER_MODE) && (MCU_ENTER_LOW_POWER_MODE == STD_ON))
/**
* @brief            This function returns whether a reset occurred during standby.
* @details          This function returns whether a reset occurred during standby.
*
* @return           boolean
* @retval           TRUE        Reset occurred during standby.
* @retval           FALSE       Reset did not occurred during standby.
*
*/
boolean MC_RGM_ResetDuringStandby(void)
{
    uint32 u32RegValue;
    boolean bResetDuringStdby = FALSE;

    u32RegValue = (uint32)(pMC_RGM->RDSS & MC_RGM_RDSS_RWBITS_MASK_U32);

    /* Store the contents of RDSS. */
    if ((uint32)0U != u32RegValue)
    {
        u32StandbyResetStatus = u32RegValue;

        /* Clear all the flags from MC_RGM_RDSS */
        pMC_RGM->RDSS = u32RegValue;
    }

    if (
           (MC_RGM_FES_RES_OCCURED_DURING_STANDBY_U32 == (u32StandbyResetStatus & MC_RGM_RDSS_FES_RES_MASK_U32)) ||
           (MC_RGM_DES_RES_OCCURED_DURING_STANDBY_U32 == (u32StandbyResetStatus & MC_RGM_RDSS_DES_RES_MASK_U32))
       )
    {
        bResetDuringStdby = (boolean)TRUE;
    }

    return bResetDuringStdby;
}
    #endif /* (MCU_ENTER_LOW_POWER_MODE == STD_ON) */
#endif /* (FEATURE_RESET_DURING_STANDBY_SUPPORTED == STD_ON) */


#if (MCU_PERFORM_RESET_API == STD_ON)
/**
* @brief            This function performs a microcontroller reset.
* @details          This function performs a microcontroller reset by using the hardware feature of
*                   the microcontroller.
*                   This is a chip-wide virtual mode during which the application is not active.
*                   The system remains in this mode until all resources are available for the
*                   embedded software to take control of the device.
*
* @param[in]        MC_RGM_pConfigPtr   Pointer to the MC_RGM configuration structure.
*
* @return           void
*
*/
void MC_RGM_PerformReset(const Power_Ip_MC_RGM_ConfigType * MC_RGM_pConfigPtr)
{
    switch (MC_RGM_pConfigPtr->MC_RGM_eResetType)
    {
        case (MCU_FUNC_RESET):
        {
            /* Initiate "Functional Reset" Mode. */
            Power_Ip_MC_ME_SocTriggerResetEvent(POWER_IP_FUNC_RESET_MODE);
            break;
        }
        case (MCU_DEST_RESET):
        {
            /* Initiate "Destructive Reset" Mode. */
            Power_Ip_MC_ME_SocTriggerResetEvent(POWER_IP_DEST_RESET_MODE);
            break;
        }
        default:
        {
            /* Default case - should never enter here. */
            break;
        }
    }
}
#endif /* (MCU_PERFORM_RESET_API == STD_ON) */

#if (defined(MCU_RESET_ALTERNATE_ISR_USED) && (MCU_RESET_ALTERNATE_ISR_USED == STD_ON))
/**
* @brief            This function handles both the 'Functional' and 'Destructive' Reset
*                   Alternate Event Interrupts.
* @details          Only the following reset events support interrupt reaction:
*                       - DEBUG_FUNC (Functional Debug Reset)
*                       - EXR (External Reset)
*
* @return           void
*
*/
/** @violates @ref Mcu_MC_RGM_c_REF_5 Global declaration of function */
void MC_RGM_ResetAltInt(void)
{
    /* Functional event IRQs */
    uint32 u32FesIntResetReason       = 0U;
    uint32 u32MaskedFesIntResetReason = 0U;

#ifdef POWER_IP_DES_IRQ_SUPPORT
  #if (STD_ON == POWER_IP_DES_IRQ_SUPPORT)
    /* Destructive event IRQs (on platforms that support this feature). */
    uint32 u32DesIntResetReason = 0U;
  #endif
#endif

    /* Get only the bits that can generate a functional reset IRQ. */
    u32FesIntResetReason = ((uint32)(pMC_RGM->FES & MC_RGM_FES_IRQ_BITS_MASK));

    /* Clear active functional IRQs */
    pMC_RGM->FES = ((uint32)(u32FesIntResetReason & MC_RGM_FES_RWBITS_MASK32));

#ifdef POWER_IP_DES_IRQ_SUPPORT
  #if (STD_ON == POWER_IP_DES_IRQ_SUPPORT)
    /* Get only the bits that can generate a destructive reset IRQ. */
    u32DesIntResetReason = ((uint32)(pMC_RGM->DES & MC_RGM_DES_IRQ_BITS_MASK));

    /* Clear active destructive IRQs */
    pMC_RGM->DES = ((uint32)(u32DesIntResetReason & MC_RGM_DES_RWBITS_MASK32));
  #endif
#endif

    if (POWER_MC_RGM_UNINIT != Power_MC_RGM_Status)
    {
        /* Check for spurious interrupt. */
        u32MaskedFesIntResetReason = u32FesIntResetReason & ( (uint32)(pMC_RGM->FERD & MC_RGM_FERD_RWBITS_MASK) );
    
        /* Signal the functional alternate reset event (IRQ) to upper layers. */
        if ((uint32)0U != u32MaskedFesIntResetReason)
        {
            ReportPowerErrors(POWER_IP_ERROR_ISR_NOTIFICATION, POWER_E_ISR_FUNC_RESET_ALT_FAILURE);
        }
        
#ifdef POWER_IP_DES_IRQ_SUPPORT
  #if (STD_ON == POWER_IP_DES_IRQ_SUPPORT)
        /* Signal the destructive alternate reset event (IRQ) to upper layers. */
        if ((uint32)0U != u32DesIntResetReason)
        {
            ReportPowerErrors(POWER_IP_ERROR_ISR_NOTIFICATION, POWER_E_ISR_DEST_RESET_ALT_FAILURE);
        }
  #endif
#endif
    }
}
#endif /* ( MCU_RESET_ALTERNATE_ISR_USED == STD_ON ) */


#define MCU_STOP_SEC_CODE
/**
* @violates @ref Mcu_MC_RGM_c_REF_1 #include directives should only be preceded by preprocessor
* directives or comments.
*
* @violates @ref Mcu_MC_RGM_c_REF_2 Precautions shall be taken in order to prevent the contents
* of a header file being included more than once.
*/
#include "Mcu_MemMap.h"

#ifdef __cplusplus
}
#endif

/** @} */

